==> landers.angular-1.0.50.tgz: package/app.js <==
angular.module('Landers.app', [])
    .provider('AngularApp', function () {
        var provider = this;

        this.appPath = null;
        this.needCss = false;

        this.$get = ['$state', 'Ajax', 'Runtime', 'AngularModel', 'Dialog', '$templateRequest', 'Flat', '$timeout', 'InjectService', '$compile', function ($state, Ajax, Runtime, AngularModel, Dialog, $templateRequest, Flat, $timeout, InjectService, $compile) {
            var defaults = {
                dialog : {
                    width : 800,
                    height : '100%',
                    scroll : true
                },
                partials : {
                    buttons : true,
                    searcher : true,
                    addedit : true
                }
            };

            function AngularApp(scope, config) {
                config = $.extend(true, {}, defaults, config);

                //初始化全局变量
                var that = this,
                    doms = [];

                this.scope = scope;
                scope.$app = scope.$app || {};
                scope.$app[config.key] = this;

                //初始化 key, name
                this.key = config.key;
                this.name = config.name;

                //初始化path
                this.path = provider.appPath + '/' + (config.path || config.key) + '/';

                //加载css
                if (provider.needCss) {
                    doms.push(Landers.loader.css(this.path + 'style.css'));
                }

                //初始化ajax
                this.ajax = Ajax.make();

                //初始化runtime (挂载$runtime至scope)
                this.runtime = Runtime.make(scope);

                this.$ = {
                    _datkey : function(objectKey, subkey){
                        var datkey = ['$' + objectKey];
                        if (subkey) datkey.push(subkey);
                        return datkey.join('.');
                    },
                    get : function(objectKey, subkey) {
                        var datkey = this._datkey(objectKey, subkey);
                        return Flat.get(scope, datkey);
                    },
                    set : function(objectKey, data, subkey) {
                        var datkey = this._datkey(objectKey);
                        Flat.set(scope, datkey, data, subkey);
                        if (!scope.$$phase) scope.$apply();
                    },
                    extend : function(objectKey, data) {
                        var datkey = this._datkey(objectKey);
                        Flat.extend(scope, data, datkey);
                        if (!scope.$$phase) scope.$apply();
                    }
                };

                //对话框
                this.dialog = {
                    dialogs : [],
                    _attachConfig : function(_dialog_config){
                        var dialog_config = angular.extend({}, _dialog_config);
                        if (_dialog_config.event) {
                            if (that.model) {
                                that.model.listitem.active(_dialog_config.event, true);
                            }
                            dialog_config.onclose = function(){
                                if (that.model) {
                                    that.model.listitem.active(_dialog_config.event, false);
                                }
                                _dialog_config.onclose && _dialog_config.onclose.call(this);
                            };
                            delete dialog_config.event;
                        }
                        dialog_config.blur = false;
                        return dialog_config;
                    },
                    _loadPartial : function(dialog_config, callback){
                        var partial, templateUrl;
                        if ( dialog_config.component_partial ) {
                            partial = dialog_config.component_partial;
                            templateUrl = scope.$partial('dialog-' + partial, true);
                        } else if ( dialog_config.partial || dialog_config.tplkey ) {
                            partial = dialog_config.partial || dialog_config.tplkey;
                            templateUrl = scope.$partial('dialog-' + partial);
                        } else {
                            throw ('参数错误');
                        }
                        $templateRequest(templateUrl).then(function(html){
                            callback(html);
                        });
                    },
                    show : function(dialog_config){
                        dialog_config = this._attachConfig(dialog_config);
                        var _this = this;
                        _this._loadPartial(dialog_config, function(html){
                            dialog_config.html = html;
                            var dialog = Dialog.show(scope, dialog_config);
                            _this.dialogs.push(dialog);
                        });
                    },
                    confirmWithPartial : function(dialog_config, callbacks){
                        dialog_config = this._attachConfig(dialog_config);
                        var _this = this;
                        _this._loadPartial(dialog_config, function(html){
                            dialog_config.html = html;
                            var dialog = Dialog.confirm(scope, dialog_config, callbacks);
                            _this.dialogs.push(dialog);
                        });
                    },
                    // 传入要显示的内容和按钮的回调
                    confirm : function(content, callbacks, $event){
                        callbacks.yesFn = callbacks.yesFn || function () {};
                        callbacks.noFn = callbacks.noFn || function () {};

                        // 处理content内容
                        content = content.replace(/&/g, "&amp;")
                            .replace(/</g, "&lt;")
                            .replace(/>/g, "&gt;");

                        dialog_config = this._attachConfig({
                            content: '<div class=\"confirm\">' + content + '</div>',
                            title: '来自'+window.location.host+'的消息',
                            width: 300,
                            event: $event
                        });

                        var dialog = Dialog.confirm(scope, dialog_config, callbacks);
                        this.dialogs.push(dialog);
                    }
                };

                //初始化model
                if (config.model !== false) {
                    var model_config = Landers.utils.json.slice(config, [
                        'key', 'name', 'path', 'api_url', 'url_style', 'url_params', 'ajax', 'vice'
                    ]);
                    model_config.ajax = model_config.ajax || that.ajax;
                    this.model = config.model || AngularModel.make(scope, model_config);

                    this.action = {
                        loading:function(){
                            if (that.ajax) that.ajax.loading();
                            return this;
                        },
                        post : function(action, data, params) {
                            var apiurl = that.model.api.buildUrl('custom', {action:action});
                            if (!data && that.model) data = that.model.getData('$edit');
                            if (that.ajax) return that.ajax.post(apiurl, data, params);
                        },
                        get : function(action, params) {
                            var apiurl = that.model.api.buildUrl('custom', {action:action});
                            if (that.ajax) return that.ajax.get(apiurl, params);
                        }
                    };
                    this.hooks = that.model.hooks;

                    //复制于lwap.js
                    this.list = that.model.list;
                    this.listitem = that.model.listitem;
                    this.addedit = angular.extend(that.model.addedit, {
                        showCreate : scope.$events.showCreate,
                        showEdit : scope.$events.showEdit
                    });
                }

                this.injectClient = function(name, inject, callback, key){
                    var path = that.path;
                    var url = path + 'client-' + name + '.js';
                    if (inject) {
                        InjectService.module(url, inject, callback);
                    } else {
                        InjectService.module(url, callback);
                    }
                };

                //与内置默认$services合并
                scope.$service = scope.$service || {
                    tagsInput : function(fields){
                        if (!angular.isArray(fields)) fields = [fields];
                        angular.forEach(fields, function(item){
                            var data = that.model.getData('$edit');
                            that.runtime.set(item, {
                                data : data[item] ? data[item].split(',') : [],
                                onChange:function(){
                                    data[item] = this.data.map(function(item){
                                        return item.text;
                                    }).join(',');
                                }
                            });
                        });
                    }
                };

                if ( !config.vice ) {
                    //扩展或覆盖crud中scope.$events
                    scope.$events = angular.extend({} , scope.$events, {
                        showEdit:function($event, item){
                            $timeout(function() {
                                config.dialog.name = config.name;
                                config.dialog.partial = config.dialog.partial || 'addedit';
                                that.addedit.popup($event, item, config.dialog);
                            }, 100);
                        },
                        showCreate:function($event){
                            this.showEdit($event);
                        }
                    });

                    //初始化
                    scope.$errors = {};

                    //scope销毁时事件
                    scope.$on('$destroy', function(){
                        //断开所有连接
                        that.ajax.cancelAll();
                        delete that.ajax;

                        //清除doms
                        angular.forEach(doms, function(dom){
                            dom.remove();
                        });

                        // Landers.dialog.clear && Landers.dialog.clear();
                        angular.forEach(that.dialog.dialogs, function(dialog){
                            if (dialog && dialog.close) {
                                dialog.close();
                            }
                        });

                        that.scope = null;

                        Landers.response.hide();
                    });
                }
            }

            return {
                make : function(scope, config){
                    return new AngularApp(scope, config);
                },
                cache : {},
                launch:function(appkey) {
                    $state.go('base.' + appkey);
                },
                loadTemplates:function(paths, callback){
                    if (paths && !angular.isArray(paths)) paths = [paths];
                    if (!paths || !paths.length) return callback();
                    var promises = [];
                    angular.forEach(paths, function(path){
                        var defer = $q.defer();
                        promises.push(defer.promise);
                        (function(defer){
                            $templateRequest(path).then(function(tpl){
                                defer.resolve(tpl);
                            });
                        })(defer);
                    });
                    $q.all(promises).then(callback);
                },
                loadAppPartial : function(key2, is_private, partial, callback) {
                    var url = $rootScope.$path.apptpl(key2, is_private, partial);
                    this.loadTemplates(url, callback);
                },
                getAppSourcesUrl : function(key, is_private){
                    var base_path = provider.appPath + key;
                    return [
                        base_path + '/style.css',
                        base_path + '/client.js'
                    ];
                },
                injectedModules:{},
                injectModule:function(key2, inject, callback){
                    var module = this.injectedModules[key2];
                    if (!module) {
                        var _this = this, urls = this.getAppSourcesUrl(key2);
                        InjectService.module(urls, inject, function(module){
                            _this.injectedModules[key2] = module;
                            callback(module);
                        });
                    } else {
                        callback(module);
                    }
                },
                injectedApps:{},
                bootApp : function(scope, key, callback_controller, is_private) {
                    var _this = this, urls = this.getAppSourcesUrl(key, is_private);
                    if (!_this.injectedApps[key]) {
                        InjectService.urls(urls, function(){
                            if (angular.isString(callback_controller)) {
                                _this.bootController(scope, callback_controller);
                            }
                            if (angular.isFunction(callback_controller)){
                                var base_path = provider.appPath + key;
                                var main_url = base_path + '/layout.html';
                                _this.bootController(scope, main_url);
                                $timeout(callback_controller, 200);
                            }
                            _this.injectedApps[key] = true;
                        });
                    } else {
                        if (angular.isFunction(callback_controller)){
                            callback_controller && callback_controller();
                        }
                    }
                },
                injectedControllers:{},
                bootController: function (scope, controller_url, reload){
                    if (!this.injectedControllers[controller_url] || reload) {
                        var html = controller_url.indexOf('/') >= 0 ?
                                   '<div ng-include="\'' + controller_url + '\'"></div>' :
                                   '<div ng-controller="' + controller_url + '"></div>';
                        var el = $compile(html)(scope);
                        angular.element('#lwap-body').append(el);
                        if (!scope.$$phase) scope.$apply();
                        this.injectedControllers[controller_url] = true;
                    };
                },
                reset : function() {
                    this.injectedControllers = {};
                    this.injectedApps = {};
                    this.injectedModules = {};
                }
            };
        }];
    });

==> landers.angular-1.0.51.tgz: package/app.js <==
angular.module('Landers.app', [])
    .provider('AngularApp', function () {
        var provider = this;

        this.appPath = null;
        this.needCss = false;

        this.$get = ['$state', 'Ajax', 'Runtime', 'AngularModel', 'Dialog', '$templateRequest', 'Flat', '$timeout', 'InjectService', '$compile', function ($state, Ajax, Runtime, AngularModel, Dialog, $templateRequest, Flat, $timeout, InjectService, $compile) {
            var defaults = {
                dialog : {
                    width : 800,
                    height : '100%',
                    scroll : true
                },
                partials : {
                    buttons : true,
                    searcher : true,
                    addedit : true
                }
            };

            function AngularApp(scope, config) {
                config = $.extend(true, {}, defaults, config);

                //初始化全局变量
                var that = this,
                    doms = [];

                this.scope = scope;
                scope.$app = scope.$app || {};
                scope.$app[config.key] = this;

                //初始化 key, name
                this.key = config.key;
                this.name = config.name;

                //初始化path
                this.path = provider.appPath + '/' + (config.path || config.key) + '/';

                //加载css
                if (provider.needCss) {
                    doms.push(Landers.loader.css(this.path + 'style.css'));
                }

                //初始化ajax
                this.ajax = Ajax.make();

                //初始化runtime (挂载$runtime至scope)
                this.runtime = Runtime.make(scope);

                this.$ = {
                    _datkey : function(objectKey, subkey){
                        var datkey = ['$' + objectKey];
                        if (subkey) datkey.push(subkey);
                        return datkey.join('.');
                    },
                    get : function(objectKey, subkey) {
                        var datkey = this._datkey(objectKey, subkey);
                        return Flat.get(scope, datkey);
                    },
                    set : function(objectKey, data, subkey) {
                        var datkey = this._datkey(objectKey);
                        Flat.set(scope, datkey, data, subkey);
                        if (!scope.$$phase) scope.$apply();
                    },
                    extend : function(objectKey, data) {
                        var datkey = this._datkey(objectKey);
                        Flat.extend(scope, data, datkey);
                        if (!scope.$$phase) scope.$apply();
                    }
                };

                //对话框
                this.dialog = {
                    dialogs : [],
                    _attachConfig : function(_dialog_config){
                        var dialog_config = angular.extend({}, _dialog_config);
                        if (_dialog_config.event) {
                            if (that.model) {
                                that.model.listitem.active(_dialog_config.event, true);
                            }
                            dialog_config.onclose = function(){
                                if (that.model) {
                                    that.model.listitem.active(_dialog_config.event, false);
                                }
                                _dialog_config.onclose && _dialog_config.onclose.call(this);
                            };
                            delete dialog_config.event;
                        }
                        dialog_config.blur = false;
                        return dialog_config;
                    },
                    _loadPartial : function(dialog_config, callback){
                        var partial, templateUrl;
                        if ( dialog_config.component_partial ) {
                            partial = dialog_config.component_partial;
                            templateUrl = scope.$partial('dialog-' + partial, true);
                        } else if ( dialog_config.partial || dialog_config.tplkey ) {
                            partial = dialog_config.partial || dialog_config.tplkey;
                            templateUrl = scope.$partial('dialog-' + partial);
                        } else {
                            throw ('参数错误');
                        }
                        $templateRequest(templateUrl).then(function(html){
                            callback(html);
                        });
                    },
                    show : function(dialog_config){
                        dialog_config = this._attachConfig(dialog_config);
                        var _this = this;
                        _this._loadPartial(dialog_config, function(html){
                            dialog_config.html = html;
                            var dialog = Dialog.show(scope, dialog_config);
                            _this.dialogs.push(dialog);
                        });
                    },
                    confirmWithPartial : function(dialog_config, callbacks){
                        dialog_config = this._attachConfig(dialog_config);
                        var _this = this;
                        _this._loadPartial(dialog_config, function(html){
                            dialog_config.html = html;
                            var dialog = Dialog.confirm(scope, dialog_config, callbacks);
                            _this.dialogs.push(dialog);
                        });
                    },
                    // 传入要显示的内容和按钮的回调
                    confirm : function(content, callbacks, $event){
                        callbacks.yesFn = callbacks.yesFn || function () {};
                        callbacks.noFn = callbacks.noFn || function () {};

                        // 处理content内容
                        content = content.replace(/&/g, "&amp;")
                            .replace(/</g, "&lt;")
                            .replace(/>/g, "&gt;");

                        dialog_config = this._attachConfig({
                            content: '<div class=\"confirm\">' + content + '</div>',
                            title: '来自'+window.location.host+'的消息',
                            width: 300,
                            event: $event
                        });

                        var dialog = Dialog.confirm(scope, dialog_config, callbacks);
                        this.dialogs.push(dialog);
                    }
                };

                //初始化model
                if (config.model !== false) {
                    var model_config = Landers.utils.json.slice(config, [
                        'key', 'name', 'path', 'api_url', 'url_style', 'url_params', 'ajax', 'vice'
                    ]);
                    model_config.ajax = model_config.ajax || that.ajax;
                    model_config.url_params = (function(){
                        var parent_source = config.parent_source;
                        var ret = model_config.url_params || {};
                        if (parent_source && parent_source.app) {
                            ret.related_key = parent_source.app.key;
                            if ( parent_source.listitem ) {
                                ret.related_id = parent_source.listitem.id
                            }
                        };
                        return ret;
                    })(),

                    this.model = config.model || AngularModel.make(scope, model_config);

                    this.action = {
                        loading:function(){
                            if (that.ajax) that.ajax.loading();
                            return this;
                        },
                        post : function(action, data, params) {
                            var apiurl = that.model.api.buildUrl('custom', {action:action});
                            if (!data && that.model) data = that.model.getData('$edit');
                            if (that.ajax) return that.ajax.post(apiurl, data, params);
                        },
                        get : function(action, params) {
                            var apiurl = that.model.api.buildUrl('custom', {action:action});
                            if (that.ajax) return that.ajax.get(apiurl, params);
                        }
                    };
                    this.hooks = that.model.hooks;

                    //复制于lwap.js
                    this.list = that.model.list;
                    this.listitem = that.model.listitem;
                    this.addedit = angular.extend(that.model.addedit, {
                        showCreate : scope.$events.showCreate,
                        showEdit : scope.$events.showEdit
                    });
                }

                this.injectClient = function(name, inject, callback, key){
                    var path = that.path;
                    var url = path + 'client-' + name + '.js';
                    if (inject) {
                        InjectService.module(url, inject, callback);
                    } else {
                        InjectService.module(url, callback);
                    }
                };

                //与内置默认$services合并
                scope.$service = scope.$service || {
                    tagsInput : function(fields){
                        if (!angular.isArray(fields)) fields = [fields];
                        angular.forEach(fields, function(item){
                            var data = that.model.getData('$edit');
                            that.runtime.set(item, {
                                data : data[item] ? data[item].split(',') : [],
                                onChange:function(){
                                    data[item] = this.data.map(function(item){
                                        return item.text;
                                    }).join(',');
                                }
                            });
                        });
                    }
                };

                if ( !config.vice ) {
                    //扩展或覆盖crud中scope.$events
                    scope.$events = angular.extend({} , scope.$events, {
                        showEdit:function($event, item){
                            $timeout(function() {
                                config.dialog.name = config.name;
                                config.dialog.partial = config.dialog.partial || 'form';
                                that.addedit.popup($event, item, config.dialog);
                            }, 100);
                        },
                        showCreate:function($event){
                            this.showEdit($event);
                        }
                    });

                    //初始化
                    scope.$errors = {};

                    //scope销毁时事件
                    scope.$on('$destroy', function(){
                        //断开所有连接
                        that.ajax.cancelAll();
                        delete that.ajax;

                        //清除doms
                        angular.forEach(doms, function(dom){
                            dom.remove();
                        });

                        // Landers.dialog.clear && Landers.dialog.clear();
                        angular.forEach(that.dialog.dialogs, function(dialog){
                            if (dialog && dialog.close) {
                                dialog.close();
                            }
                        });

                        that.scope = null;

                        Landers.response.hide();
                    });
                }
            }

            return {
                make : function(scope, config){
                    return new AngularApp(scope, config);
                },
                cache : {},
                launch:function(appkey) {
                    $state.go('base.' + appkey);
                },
                loadTemplates:function(paths, callback){
                    if (paths && !angular.isArray(paths)) paths = [paths];
                    if (!paths || !paths.length) return callback();
                    var promises = [];
                    angular.forEach(paths, function(path){
                        var defer = $q.defer();
                        promises.push(defer.promise);
                        (function(defer){
                            $templateRequest(path).then(function(tpl){
                                defer.resolve(tpl);
                            });
                        })(defer);
                    });
                    $q.all(promises).then(callback);
                },
                loadAppPartial : function(key2, is_private, partial, callback) {
                    var url = $rootScope.$path.apptpl(key2, is_private, partial);
                    this.loadTemplates(url, callback);
                },
                getAppSourcesUrl : function(key, is_private){
                    var base_path = provider.appPath + key;
                    return [
                        base_path + '/style.css',
                        base_path + '/client.js'
                    ];
                },
                injectedModules:{},
                injectModule:function(key2, inject, callback){
                    var module = this.injectedModules[key2];
                    if (!module) {
                        var _this = this, urls = this.getAppSourcesUrl(key2);
                        InjectService.module(urls, inject, function(module){
                            _this.injectedModules[key2] = module;
                            callback(module);
                        });
                    } else {
                        callback(module);
                    }
                },
                injectedApps:{},
                bootApp : function(scope, key, callback_controller, is_private) {
                    var _this = this, urls = this.getAppSourcesUrl(key, is_private);
                    if (!_this.injectedApps[key]) {
                        InjectService.urls(urls, function(){
                            if (angular.isString(callback_controller)) {
                                _this.bootController(scope, callback_controller);
                            }
                            if (angular.isFunction(callback_controller)){
                                var base_path = provider.appPath + key;
                                var main_url = base_path + '/layout.html';
                                _this.bootController(scope, main_url);
                                $timeout(callback_controller, 200);
                            }
                            _this.injectedApps[key] = true;
                        });
                    } else {
                        if (angular.isFunction(callback_controller)){
                            callback_controller && callback_controller();
                        }
                    }
                },
                injectedControllers:{},
                bootController: function (scope, controller_url, reload){
                    if (!this.injectedControllers[controller_url] || reload) {
                        var html = controller_url.indexOf('/') >= 0 ?
                                   '<div ng-include="\'' + controller_url + '\'"></div>' :
                                   '<div ng-controller="' + controller_url + '"></div>';
                        var el = $compile(html)(scope);
                        angular.element('#lwap-body').append(el);
                        if (!scope.$$phase) scope.$apply();
                        this.injectedControllers[controller_url] = true;
                    };
                },
                reset : function() {
                    this.injectedControllers = {};
                    this.injectedApps = {};
                    this.injectedModules = {};
                }
            };
        }];
    });
